//
// ********************************************************************
// * DISCLAIMER                                                       *
// *                                                                  *
// * The following disclaimer summarizes all the specific disclaimers *
// * of contributors to this software. The specific disclaimers,which *
// * govern, are listed with their locations in:                      *
// *   http://cern.ch/geant4/license                                  *
// *                                                                  *
// * Neither the authors of this software system, nor their employing *
// * institutes,nor the agencies providing financial support for this *
// * work  make  any representation or  warranty, express or implied, *
// * regarding  this  software system or assume any liability for its *
// * use.                                                             *
// *                                                                  *
// * This  code  implementation is the  intellectual property  of the *
// * GEANT4 collaboration.                                            *
// * By copying,  distributing  or modifying the Program (or any work *
// * based  on  the Program)  you indicate  your  acceptance of  this *
// * statement, and all its terms.                                    *
// ********************************************************************
// ********************************************************************

//
// --------------------------------------------------------------
//                 GEANT 4 - Brachytherapy example
// --------------------------------------------------------------
//
// Code developed by:
// S. Agostinelli, F. Foppiano, S. Garelli , M. Tropeano, S.Guatelli
//
//
//    ****************************************
//    *                                      *
//    *    BrachyDetectorConstruction.cc     *
//    *                                      *
//    ****************************************
//
// $Id: BrachyDetectorConstruction.cc,v 1.21 2003/05/22 17:20:42 guatelli Exp $
// GEANT4 tag $Name: brachy-V05-01-03 $
//
#include "BrachyPhantomROGeometry.hh"
#include "BrachyPhantomSD.hh"
#include "BrachyDetectorMessenger.hh"
#include "BrachyDetectorConstruction.hh"
#include "G4CSGSolid.hh"
#include "G4Sphere.hh"
#include "G4MaterialPropertyVector.hh"
#include "G4SDManager.hh"
#include "G4SubtractionSolid.hh"
#include "G4RunManager.hh"
#include "G4MaterialPropertiesTable.hh"
#include "G4Material.hh"
#include "G4Box.hh"
#include "G4Tubs.hh"
#include "G4LogicalVolume.hh"
#include "G4ThreeVector.hh"
#include "G4PVPlacement.hh"
#include "globals.hh"
#include "G4MaterialTable.hh"
#include "Randomize.hh"  
#include "G4RunManager.hh"
#include "G4Element.hh"
#include "G4ElementTable.hh"
#include "G4PVParameterised.hh"
#include "G4Transform3D.hh"
#include "G4RotationMatrix.hh"
#include "G4FieldManager.hh"
#include "G4TransportationManager.hh"
#include "G4SDManager.hh"
#include "G4Colour.hh"
#include "G4UserLimits.hh"
#include "G4UnionSolid.hh"
#include "G4VisAttributes.hh"
#include "G4Colour.hh"
#include "BrachyMaterial.hh"

BrachyDetectorConstruction::BrachyDetectorConstruction(G4String &SDName)
: detectorChoice(0), phantomSD(0), phantomROGeometry(0),
  World(0), WorldLog(0), WorldPhys(0),
  Phantom(0), PhantomLog(0), PhantomPhys(0),
  phantomAbsorberMaterial(0), defaultTub(0), defaultTubLog(0),
  defaultTubPhys(0),  capsule(0),capsuleLog(0),capsulePhys(0),
  capsuleTip(0),capsuleTipLog(0),capsuleTipPhys1(0),capsuleTipPhys2(0),
  iodiumCore(0),iodiumCoreLog(0),iodiumCorePhys(0),
  marker(0),markerLog(0),markerPhys(0)

{
  phantomDimensionX=30*cm ;
  phantomDimensionY=30*cm;
  phantomDimensionZ=30*cm;

  numberOfVoxelsAlongX=300;
  numberOfVoxelsAlongZ=300;

  ComputeDimVoxel();

  Worldx = 4.0*m;
  Worldy = 4.0*m;
  Worldz = 4.0*m;

  sensitiveDetectorName = SDName;

  detectorMessenger = new BrachyDetectorMessenger(this);

  pMaterial= new BrachyMaterial();
}


BrachyDetectorConstruction::~BrachyDetectorConstruction()
{ 
  delete pMaterial;
  delete detectorMessenger;
  if (phantomROGeometry) delete phantomROGeometry;
}

G4VPhysicalVolume* BrachyDetectorConstruction::Construct()
{
  pMaterial-> DefineMaterials();

  ConstructPhantom();

  ConstructSensitiveDetector();

  return WorldPhys;
}

void BrachyDetectorConstruction::ConstructPhantom()
{
  //colors' definition
  G4Colour  lblue   (0.0, 0.0, .75);
  G4Colour  red     (1.0, 0.0, 0.0) ;
  G4Colour  magenta (1.0, 0.0, 1.0) ; 
 
  //Materials for phantom
  G4Material* air = pMaterial->GetMat("Air") ;
  G4Material* soft = pMaterial->GetMat("tissue");
 
 //Get materials for source construction ...
  G4Material* titanium = pMaterial->GetMat("titanium");
  G4Material* iodium = pMaterial->GetMat("Iodium");
  G4Material* gold = pMaterial->GetMat("gold");


  ComputeDimVoxel();// Calculate voxel dimensions

  // World volume
  World = new G4Box("World",Worldx,Worldy,Worldz);
  WorldLog = new G4LogicalVolume(World,air,"WorldLog",0,0,0);
  WorldPhys = new G4PVPlacement(0,G4ThreeVector(),
                                "WorldPhys",
                                 WorldLog,
                                 0,false,0);

  // Water Box
  Phantom = new G4Box("Phantom",
                      phantomDimensionX/2,
                      phantomDimensionY/2,
                      phantomDimensionZ/2);

  PhantomLog = new G4LogicalVolume(Phantom,soft,"PhantomLog",0,0,0);
  PhantomPhys = new G4PVPlacement(0,G4ThreeVector(),
                                  "PhantomPhys",
                                  PhantomLog,
                                  WorldPhys,
                                  false,0); 

// source Bebig Isoseed I-125 ...

   defaultTub = new G4Tubs("DefaultTub",0.*mm,0.40*mm,1.84*mm,0.*deg,360.*deg);
  defaultTubLog = new G4LogicalVolume(defaultTub,air,"DefaultTub_Log");
  defaultTubPhys = new G4PVPlacement(0,
                                      G4ThreeVector(),
                                      "defaultTub_Phys",
                                      defaultTubLog,
                                      WorldPhys,
                                      false,
                                      0); 
  //  Capsule main body ...

  G4double capsuleR = 0.35*mm;
  capsule = new G4Tubs("Capsule", capsuleR,0.40*mm,1.84*mm,0.*deg,360.*deg);
  capsuleLog = new G4LogicalVolume(capsule,titanium,"CapsuleLog");
  capsulePhys = new G4PVPlacement(0,
                                  G4ThreeVector(),
                                  "CapsulePhys",
                                  capsuleLog,
                                  defaultTubPhys,
                                  false,
                                  0);
  // Capsule tips

  capsuleTip = new G4Sphere("CapsuleTip",
                            0.*mm,
                            0.40*mm,
                            0.*deg,
                            360.*deg,
                            0.*deg,
                            90.*deg);
  capsuleTipLog = new G4LogicalVolume(capsuleTip,titanium,"CapsuleTipLog");
  capsuleTipPhys1 = new G4PVPlacement(0,
                                      G4ThreeVector(0.,0.,1.84*mm),
                                      "CapsuleTipPhys1",
                                      capsuleTipLog,
                                      WorldPhys,
                                      false,
                                      0);

  G4RotationMatrix* rotateMatrix = new G4RotationMatrix();
  rotateMatrix->rotateX(180.0*deg);
  capsuleTipPhys2 = new G4PVPlacement(rotateMatrix, 
                                      G4ThreeVector(0,0,-1.84*mm),
                                      "CapsuleTipPhys2",
                                      capsuleTipLog,
                                      WorldPhys,
                                      false,
                                      0);
 
  // radiactive core ...
  iodiumCore = new G4Tubs("ICore",0.085*mm,0.35*mm,1.75*mm,0.*deg,360.*deg);
  iodiumCoreLog = new G4LogicalVolume(iodiumCore,iodium,"iodiumCoreLog");
  iodiumCorePhys = new G4PVPlacement(0,
                                     G4ThreeVector(0.,0.,0.),
                                     "iodiumCorePhys",
                                     iodiumCoreLog,
                                     defaultTubPhys,
                                     false,
                                     0);
  // golden marker
  marker = new G4Tubs("GoldenMarker",0.*mm,0.085*mm,1.75*mm,0.*deg,360.*deg);
  markerLog = new G4LogicalVolume(marker,gold,"MarkerLog");
  markerPhys = new G4PVPlacement(0,
                                 G4ThreeVector(0.,0.,0.),
                                 "MarkerPhys",
                                 markerLog,
                                 defaultTubPhys,
                                 false,
                                 0);
  // Visual attributes ...

  //World
  WorldLog->SetVisAttributes (G4VisAttributes::Invisible);

  //Phantom
  G4VisAttributes* simpleBoxVisAtt= new G4VisAttributes(lblue);
  simpleBoxVisAtt->SetVisibility(true);
  simpleBoxVisAtt->SetForceWireframe(true);
  PhantomLog->SetVisAttributes(simpleBoxVisAtt);

  //Source
  G4VisAttributes* simpleMarkerVisAtt= new G4VisAttributes(lblue);
  simpleMarkerVisAtt->SetVisibility(true);
  simpleMarkerVisAtt->SetForceSolid(true);
  markerLog->SetVisAttributes( simpleMarkerVisAtt);  

  G4VisAttributes* simpleiodiumVisAtt= new G4VisAttributes(magenta);
  simpleiodiumVisAtt->SetVisibility(true);
  simpleiodiumVisAtt->SetForceWireframe(true);
  iodiumCoreLog->SetVisAttributes(simpleiodiumVisAtt);

  G4VisAttributes* simpleCapsuleVisAtt= new G4VisAttributes(red);
  simpleCapsuleVisAtt->SetVisibility(true);  
  simpleCapsuleVisAtt->SetForceWireframe(true);
  capsuleLog->SetVisAttributes( simpleCapsuleVisAtt);

  G4VisAttributes* simpleCapsuleTipVisAtt= new G4VisAttributes(red);
  simpleCapsuleTipVisAtt->SetVisibility(true); 
  simpleCapsuleTipVisAtt->SetForceSolid(true);
  capsuleTipLog->SetVisAttributes( simpleCapsuleTipVisAtt);
}
void  BrachyDetectorConstruction::ConstructSensitiveDetector()
// Sensitive Detector and ReadOut geometry definition
{ 
  G4SDManager* pSDManager = G4SDManager::GetSDMpointer();

  if(!phantomSD)
  {
    phantomSD = new BrachyPhantomSD(sensitiveDetectorName,numberOfVoxelsAlongX,numberOfVoxelsAlongZ);
    G4String ROGeometryName = "PhantomROGeometry";
    phantomROGeometry = new BrachyPhantomROGeometry(ROGeometryName,phantomDimensionX,phantomDimensionZ,numberOfVoxelsAlongX,numberOfVoxelsAlongZ);
    phantomROGeometry->BuildROGeometry();
    phantomSD->SetROgeometry(phantomROGeometry);
    pSDManager->AddNewDetector(phantomSD);
    PhantomLog->SetSensitiveDetector(phantomSD);
  }
}

void BrachyDetectorConstruction::PrintDetectorParameters()
{
  G4cout << "-----------------------------------------------------------------------"
         << G4endl
         << "the detector is a  box whose size is: " << G4endl
         << phantomDimensionX/cm
         << " cm * "
         << phantomDimensionY/cm
         << " cm * "
         << phantomDimensionZ/cm
         << " cm" << G4endl
         << "numVoxel: "
         << numberOfVoxelsAlongX <<G4endl
         << "dim voxel: "
         << dimVoxel/mm
         << "mm" << G4endl 
         << "material of the box : "
         << phantomAbsorberMaterial->GetName() <<G4endl
         << "the source is at the center  of the detector" << G4endl
         << "-------------------------------------------------------------------------"
         << G4endl;
}

void BrachyDetectorConstruction::SetPhantomMaterial(G4String materialChoice)
{
  //Change absorber material
  // search the material by its name   
  G4Material* pttoMaterial = G4Material::GetMaterial(materialChoice);     
  if (pttoMaterial)
  {
    phantomAbsorberMaterial = pttoMaterial;
    PhantomLog->SetMaterial(pttoMaterial); 
    PrintDetectorParameters();
  } 
  else
    G4cout << "WARNING: material '" << materialChoice
           << "' not available!" << G4endl;            
}
